/*
 *	SCCS: @(#)syscall.c	1.5 (97/07/21)
 *
 *	UniSoft Ltd., London, England
 *
 * (C) Copyright 1996 X/Open Company Limited
 *
 * All rights reserved.  No part of this source code may be reproduced,
 * stored in a retrieval system, or transmitted, in any form or by any
 * means, electronic, mechanical, photocopying, recording or otherwise,
 * except as stated in the end-user licence agreement, without the prior
 * permission of the copyright owners.
 * A copy of the end-user licence agreement is contained in the file
 * Licence which accompanies this distribution.
 * 
 * X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
 * the UK and other countries.
 */

#ifndef lint
static char sccsid[] = "@(#)syscall.c	1.5 (97/07/21) TET3 release 3.6";
#endif

/************************************************************************

SCCS:   	@(#)syscall.c	1.5 97/07/21 TETware release 3.6
NAME:		syscall.c
PRODUCT:	TETware
AUTHOR:		Andrew Dingwall, UniSoft Ltd.
DATE CREATED:	August 1996

DESCRIPTION:
	functions which look like system calls

	in TETware-Lite, each function invokes a system call on the
	local system

	in fully-featured TETware, each function invokes a server
	interface function which instructs a TCCD to perform the
	required action

MODIFICATIONS:

************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <errno.h>
#  include <sys/stat.h>
#    include <unistd.h>
#    include <signal.h>
#    include <sys/wait.h>
#include "dtmac.h"
#include "dtmsg.h"
#include "error.h"
#include "servlib.h"
#include "dtetlib.h"
#include "tcc.h"
#include "tcclib.h"

#ifndef NOTRACE
#include "ltoa.h"
#endif

/* mode for created directories */
#  define MODEANY	((mode_t) (S_IRWXU | S_IRWXG | S_IRWXO))

int tet_tcerrno;	/* fake TCCD reply code */

#ifdef NEEDsrcFile
static char srcFile[] = __FILE__;	/* file name for error reporting */
#endif

/*
**	all these functions set tet_tcerrno on return
**
**	if an error is returned:
**		if the error is a system error, errno is set to indicate
**		the cause of the error;
**		otherwise, errno is set to 0 and tet_tcerrno is set to
**		indicate the cause of the error
*/


/* ARGSUSED */
int tcc_access(sysid, path, mode)
int sysid, mode;
char *path;
{
	register int rc;

	rc = tet_eaccess(path, mode);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

/* ARGSUSED */
int tcc_mkdir(sysid, dir)
int sysid;
char *dir;
{
	register int rc;

	rc = tet_mkdir(dir, MODEANY);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

/* ARGSUSED */
int tcc_rmdir(sysid, dir)
int sysid;
char *dir;
{
	register int rc;

	rc = tet_rmdir(dir);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

/* ARGSUSED */
int tcc_chdir(sysid, dir)
int sysid;
char *dir;
{
	register int rc;

	rc = CHDIR(dir);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

/* ARGSUSED */
int tcc_unlink(sysid, fname)
int sysid;
char *fname;
{
	register int rc;

	rc = UNLINK(fname);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

int tcc_kill(sysid, pid, signum)
int sysid, signum;
long pid;
{
	register int rc;

	TRACE4(tet_Ttcc, 4, "sending signal %s to pid %s on system %s",
		tet_i2a(signum), tet_l2a(pid), tet_i2a(sysid));

	rc = KILL(pid, signum);
	tet_tcerrno = rc < 0 ? tet_maperrno(errno) : ER_OK;

	return(rc);
}

int tcc_waitnohang(sysid, remid, statp)
int sysid, *statp;
long remid;
{
	register int rc;
	register int errsave;

	pid_t pid;
	int status;

	TRACE3(tet_Ttcc, 4, "wait for pid %s on system %s",
		tet_l2a(remid), tet_i2a(sysid));



	if ((pid = waitpid((pid_t) remid, &status, WNOHANG)) == (pid_t) 0) {
		tet_tcerrno = ER_WAIT;
		rc = -1;
	}
	else if (pid == (pid_t) -1) {
		tet_tcerrno = (errno == ECHILD) ? ER_PID : ER_ERR;
		rc = -1;
	}
	else {
		ASSERT(pid == (pid_t) remid);
		tet_tcerrno = ER_OK;
		*statp = tet_mapstatus(status);
		rc = 0;
	}



	errsave = errno;

	TRACE4(tet_Ttcc, 4, "wait returns %s, %s = %s", tet_i2a(rc),
		rc < 0 ? "tet_tcerrno" : "status",
		rc < 0 ? tet_ptrepcode(tet_tcerrno) : tet_i2x(*statp));

	errno = errsave;
	return(rc);
}

